package org.jtheque.films;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.update.IUpdateManager;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.managers.error.InternationalizedError;
import org.jtheque.core.managers.error.IErrorManager;
import org.jtheque.core.managers.feature.Feature;
import org.jtheque.core.managers.feature.IFeatureManager;
import org.jtheque.core.managers.feature.IFeatureManager.CoreFeature;
import org.jtheque.core.managers.feature.Feature.FeatureType;
import org.jtheque.core.managers.file.IFileManager;
import org.jtheque.core.managers.file.able.FileType;
import org.jtheque.core.managers.module.beans.CollectionBasedModule;
import org.jtheque.core.managers.module.annotations.ModuleDefinition;
import org.jtheque.core.managers.module.annotations.PrimaryModule;
import org.jtheque.core.managers.module.annotations.Module;
import org.jtheque.core.managers.module.annotations.PrePlug;
import org.jtheque.core.managers.module.annotations.Plug;
import org.jtheque.core.managers.module.annotations.UnPlug;
import org.jtheque.core.managers.schema.Schema;
import org.jtheque.core.managers.schema.ISchemaManager;
import org.jtheque.core.managers.state.StateException;
import org.jtheque.core.managers.state.IStateManager;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.core.managers.view.able.components.TabComponent;
import org.jtheque.core.managers.view.impl.components.config.ConfigTabComponent;
import org.jtheque.films.persistence.FilmsSchema;
import org.jtheque.core.utils.ui.constraints.ConstraintManager;
import org.jtheque.core.utils.ui.constraints.MaxLenghtConstraint;
import org.jtheque.core.utils.ui.constraints.NotNullConstraint;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.primary.services.able.ICollectionsService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.services.impl.utils.config.ConfigManager;
import org.jtheque.films.services.impl.utils.config.Configuration;
import org.jtheque.films.services.impl.utils.file.backup.JTDBackupWriter;
import org.jtheque.films.services.impl.utils.file.backup.XMLBackupWriter;
import org.jtheque.films.services.impl.utils.file.restore.DBV3BackupReader;
import org.jtheque.films.services.impl.utils.file.restore.DBV4BackupReader;
import org.jtheque.films.services.impl.utils.file.restore.JTDBackupReader;
import org.jtheque.films.services.impl.utils.file.restore.XMLBackupReader;
import org.jtheque.films.services.impl.utils.web.GettersUpdatable;
import org.jtheque.films.utils.Constants;
import org.jtheque.films.utils.Constants.Properties.Collection;
import org.jtheque.films.utils.Constants.Properties.Film;
import org.jtheque.films.utils.Constants.Properties.Person;
import org.jtheque.films.utils.Constants.Properties.Publication;
import org.jtheque.films.controllers.able.IFilmController;
import org.jtheque.primary.PrimaryUtils;
import org.jtheque.primary.services.able.ITypesService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.primary.services.able.ISagasService;
import org.jtheque.primary.utils.DataTypeManager;
import org.jtheque.primary.view.impl.choice.ChoiceAction;
import org.jtheque.primary.view.impl.choice.ChoiceActionFactory;
import org.jtheque.primary.view.impl.sort.Sorter;
import org.jtheque.primary.view.impl.sort.SorterFactory;

/**
 * A JTheque Module for managing films.
 *
 * @author Baptiste Wicht
 */
@Module(name = "JTheque Films Module", author = "Baptiste Wicht", description = "Module de gestion de films", 
        version = "1.3", coreVersion = "2.0.1", jarFile = "jtheque-films-module-1.4.jar")
@PrimaryModule(icon = "org/jtheque/films/ressources/images/FilmsModule.png")
@ModuleDefinition(updateURL = "http://jtheque.developpez.com/public/versions/FilmsModule.versions")
public class FilmsModule implements CollectionBasedModule, IFilmsModule {
    private ConfigManager config;

    private ConfigTabComponent[] configTabComponents;
    private TabComponent[] tabComponents;
    private Sorter[] sorters;
    private ChoiceAction[] choiceActions;

    //Features
    private Feature sitesFeature;
    private Feature refreshFeature;
    private Feature printFeature;
    private Feature exportFeature;
    private Feature importFolderFeature;
    private Feature importFeature;
    private Feature searchFeature;
    private Feature filmsFeature;
    private Feature actorsFeature;
    private Feature realizersFeature;
    private Feature lendingsFeature;
    private Feature othersFeature;
    private Feature publicationFeature;
    private Feature videoFeature;
    private Feature coverFeature;
    
    //Writers/Readers
    private JTDBackupWriter jtdWriter;
    private XMLBackupWriter xmlWriter;
    private XMLBackupReader xmlReader;
    private JTDBackupReader jtdReader;
    private DBV3BackupReader dbv3Reader;
    private DBV4BackupReader dbv4Reader;

    private static final String BASENAME = "org.jtheque.films.ressources.i18n.films";

    private Schema schema;

    @PrePlug
    public void prePlug() {
        Managers.getManager(ILanguageManager.class).addBasename(BASENAME);

        Managers.getManager(IUpdateManager.class).registerUpdatable(GettersUpdatable.getInstance());

        PrimaryUtils.setPrimaryImpl("Films");
        PrimaryUtils.prePlug();

        schema = new FilmsSchema();

        Managers.getManager(ISchemaManager.class).registerSchema(schema);
    }

    @Plug
    public void plug() {
        loadConfiguration();

        PrimaryUtils.plug();

        DataTypeManager.bindDataTypeToKey(IFilmsService.DATA_TYPE, "film.data.title");
        DataTypeManager.bindDataTypeToKey(IActorService.DATA_TYPE, "actor.data.title");
        DataTypeManager.bindDataTypeToKey(IRealizersService.DATA_TYPE, "realizer.data.title");
        DataTypeManager.bindDataTypeToKey(ITypesService.DATA_TYPE, "type.data.title");
        DataTypeManager.bindDataTypeToKey(IKindsService.DATA_TYPE, "kind.data.title");
        DataTypeManager.bindDataTypeToKey(ISagasService.DATA_TYPE, "saga.data.title");
        
        for (Sorter sorter : sorters) {
            SorterFactory.getInstance().addSorter(sorter);
        }
        
        addFeatures();
        
        //Config panels

        for(ConfigTabComponent component : configTabComponents){
            Managers.getManager(IViewManager.class).addConfigTabComponent(component);
        }
    }

    private void loadConfiguration() {
        config = Managers.getManager(IStateManager.class).getState(ConfigManager.class);

        if (config == null) {
            try {
                config = Managers.getManager(IStateManager.class).createState(ConfigManager.class);
            } catch (StateException e) {
                Managers.getManager(ILoggingManager.class).getLogger(getClass()).exception(e);
                config = new ConfigManager();
                Managers.getManager(IErrorManager.class).addError(new InternationalizedError("error.loading.configuration"));
            }

            config.setDefaults();
        }
    }

    private void addFeatures() {
        //Help

        IFeatureManager manager = Managers.getManager(IFeatureManager.class);

        sitesFeature = manager.addSubFeature(manager.getFeature(CoreFeature.HELP), "openSitesViewAction", FeatureType.ACTION, 2);

        //File

        refreshFeature = manager.addSubFeature(manager.getFeature(CoreFeature.FILE), "refreshListAction", FeatureType.ACTION, 100);

        printFeature = manager.addSubFeature(manager.getFeature(CoreFeature.FILE), "printListAction", FeatureType.ACTION, 101);

        publicationFeature = manager.addSubFeature(manager.getFeature(CoreFeature.FILE), "openPublicationViewAction", FeatureType.ACTION, 102);

        addExportFeatures(manager);

        addImportFeatures(manager);

        coverFeature = manager.addSubFeature(manager.getFeature(CoreFeature.FILE), "openCoverViewAction", FeatureType.SEPARATED_ACTION, 105);

        importFolderFeature = manager.addSubFeature(manager.getFeature(CoreFeature.FILE), "openAutoImportViewAction", FeatureType.ACTION, 106);

        searchFeature = manager.createFeature(100, FeatureType.ACTIONS, "menu.jtheque.edit.find");

        manager.addSubFeature(searchFeature, "searchFilmAction", FeatureType.ACTION, 1);
        manager.addSubFeature(searchFeature, "searchActorAction", FeatureType.ACTION, 2);
        manager.addSubFeature(searchFeature, "searchRealizerAction", FeatureType.ACTION, 3);

        manager.getFeature(CoreFeature.EDIT).addSubFeature(searchFeature);

        //Films

        filmsFeature = manager.createFeature(100, FeatureType.PACK, "menu.jtheque.films");

        manager.addSubFeature(filmsFeature, "newFilmAction", FeatureType.ACTION, 1, Constants.IMAGE_BASENAME, "add_film");

        manager.addFeature(filmsFeature);

        //Actors

        actorsFeature = manager.createFeature(200, FeatureType.PACK, "menu.jtheque.actors");

        manager.addSubFeature(actorsFeature, "newActorAction", FeatureType.ACTION, 1, Constants.IMAGE_BASENAME, "add_perso");

        manager.addFeature(actorsFeature);

        //Realizers

        realizersFeature = manager.createFeature(300, FeatureType.PACK, "menu.jtheque.realizers");

        manager.addSubFeature(realizersFeature, "newRealizerAction", FeatureType.ACTION, 1, Constants.IMAGE_BASENAME, "add_perso");

        manager.addFeature(realizersFeature);

        //Lendings

        lendingsFeature = manager.createFeature(400, FeatureType.PACK, "menu.jtheque.lendings");

        manager.addSubFeature(lendingsFeature, "openLendingsViewAction", FeatureType.ACTION, 1, Constants.IMAGE_BASENAME, "prets");
        manager.addSubFeature(lendingsFeature, "lendFilmAction", FeatureType.ACTION, 2);
        manager.addSubFeature(lendingsFeature, "returnFilmAction", FeatureType.ACTION, 3);

        manager.addFeature(lendingsFeature);

        //Others

        addOthersFeature(manager);

        videoFeature = manager.addSubFeature(manager.getFeature(CoreFeature.ADVANCED), "openVideoViewAction", FeatureType.SEPARATED_ACTION, 500);
    }

    private void addExportFeatures(IFeatureManager manager) {
        exportFeature = manager.createFeature(103, FeatureType.SEPARATED_ACTIONS, "menu.jtheque.file.export");
        exportFeature.setBaseName(Constants.IMAGE_BASENAME);
        exportFeature.setIcon("export");

        manager.addSubFeature(exportFeature, "exportToExcelAction", FeatureType.ACTION, 1);
        manager.addSubFeature(exportFeature, "exportToXMLAction", FeatureType.ACTION, 2);
        manager.addSubFeature(exportFeature, "exportToHTMLAction", FeatureType.ACTION, 3);
        manager.addSubFeature(exportFeature, "exportToPDFAction", FeatureType.ACTION, 4);
        manager.addSubFeature(exportFeature, "exportToTXTAction", FeatureType.ACTION, 5);
        manager.addSubFeature(exportFeature, "exportToJTFEAction", FeatureType.ACTION, 6);
        manager.addSubFeature(exportFeature, "exportToCSVAction", FeatureType.ACTION, 7);
        manager.addSubFeature(exportFeature, "exportToRTFAction", FeatureType.ACTION, 8);

        manager.getFeature(CoreFeature.FILE).addSubFeature(exportFeature);
    }

    private void addImportFeatures(IFeatureManager manager) {
        importFeature = manager.createFeature(104, FeatureType.ACTIONS, "menu.jtheque.file.import");
        importFeature.setBaseName(Constants.IMAGE_BASENAME);
        importFeature.setIcon("import");

        manager.addSubFeature(importFeature, "importFromXMLAction", FeatureType.ACTION, 1);
        manager.addSubFeature(importFeature, "importFromJTFAction", FeatureType.ACTION, 2);
        manager.addSubFeature(importFeature, "importFromJTFEAction", FeatureType.ACTION, 3);

        manager.getFeature(CoreFeature.FILE).addSubFeature(importFeature);
    }

    private void addOthersFeature(IFeatureManager manager) {
        othersFeature = manager.createFeature(500, FeatureType.PACK, "menu.jtheque.others");

        Feature newFeature = manager.createFeature(1, FeatureType.ACTIONS, "menu.jtheque.others.new");
        newFeature.setBaseName(Constants.IMAGE_BASENAME);
        newFeature.setIcon("add_others");

        manager.addSubFeature(newFeature, "newKindAction", FeatureType.ACTION, 1);
        manager.addSubFeature(newFeature, "newTypeAction", FeatureType.ACTION, 2);
        manager.addSubFeature(newFeature, "newLanguageAction", FeatureType.ACTION, 3);
        manager.addSubFeature(newFeature, "newBorrowerMenuAction", FeatureType.ACTION, 4);
        manager.addSubFeature(newFeature, "newCountryMenuAction", FeatureType.ACTION, 5);
        manager.addSubFeature(newFeature, "newSagaAction", FeatureType.ACTION, 6);

        othersFeature.addSubFeature(newFeature);

        Feature deleteFeature = manager.createFeature(2, FeatureType.ACTIONS, "menu.jtheque.others.delete");
        deleteFeature.setBaseName(Constants.IMAGE_BASENAME);
        deleteFeature.setIcon("delete_others");

        manager.addSubFeature(deleteFeature, "deleteKindMenuAction", FeatureType.ACTION, 1);
        manager.addSubFeature(deleteFeature, "deleteTypeMenuAction", FeatureType.ACTION, 2);
        manager.addSubFeature(deleteFeature, "deleteLanguageMenuAction", FeatureType.ACTION, 3);
        manager.addSubFeature(deleteFeature, "deleteCountryMenuAction", FeatureType.ACTION, 4);
        manager.addSubFeature(deleteFeature, "deleteBorrowerMenuAction", FeatureType.ACTION, 5);
        manager.addSubFeature(deleteFeature, "deleteSagaMenuAction", FeatureType.ACTION, 6);

        othersFeature.addSubFeature(deleteFeature);

        Feature editFeature = manager.createFeature(3, FeatureType.ACTIONS, "menu.jtheque.others.modify");
        editFeature.setBaseName(Constants.IMAGE_BASENAME);
        editFeature.setIcon("edit_others");

        manager.addSubFeature(editFeature, "editKindMenuAction", FeatureType.ACTION, 1);
        manager.addSubFeature(editFeature, "editTypeMenuAction", FeatureType.ACTION, 2);
        manager.addSubFeature(editFeature, "editLanguageMenuAction", FeatureType.ACTION, 3);
        manager.addSubFeature(editFeature, "editBorrowerMenuAction", FeatureType.ACTION, 5);
        manager.addSubFeature(editFeature, "editCountryMenuAction", FeatureType.ACTION, 4);
        manager.addSubFeature(editFeature, "editSagaMenuAction", FeatureType.ACTION, 6);

        othersFeature.addSubFeature(editFeature);

        manager.addFeature(othersFeature);
    }

    @Override
    public boolean chooseCollection(String collection, String password, boolean create) {
        ICollectionsService collectionsService = (ICollectionsService)Managers.getManager(IBeansManager.class).getBean("collectionsService");
        
        if(create){
            collectionsService.createCollectionAndUse(collection, password);
        } else if(!collectionsService.login(collection, password)){
            return false;
        }
        
        return true;
    }

    @Override
    public void plugCollection() {
        for (ChoiceAction action : choiceActions) {
            ChoiceActionFactory.addChoiceAction(action);
        }

        for(TabComponent component : tabComponents){
            Managers.getManager(IViewManager.class).addTabComponent(component);
        }

        configureBackupAndRestore();

        configureViewConstraints();

        ((IFilmController)Managers.getManager(IBeansManager.class).getBean("filmController")).getView().selectFirst();
    }

    private void configureBackupAndRestore() {
        jtdWriter = new JTDBackupWriter();
        xmlWriter = new XMLBackupWriter();

        xmlReader = new XMLBackupReader();
        jtdReader = new JTDBackupReader();
        dbv3Reader = new DBV3BackupReader();
        dbv4Reader = new DBV4BackupReader();

        Managers.getManager(IFileManager.class).registerBackupWriter(FileType.JTD, jtdWriter);
        Managers.getManager(IFileManager.class).registerBackupWriter(FileType.XML, xmlWriter);

        Managers.getManager(IFileManager.class).registerBackupReader(FileType.XML, xmlReader);
        Managers.getManager(IFileManager.class).registerBackupReader(FileType.JTD, jtdReader);
        Managers.getManager(IFileManager.class).registerBackupReader(FileType.V3, dbv3Reader);
        Managers.getManager(IFileManager.class).registerBackupReader(FileType.V4, dbv4Reader);
    }

    private void configureViewConstraints() {
        ConstraintManager.addConstraint(Person.NAME, new MaxLenghtConstraint(100, Person.NAME, false, false));
        ConstraintManager.addConstraint(Person.FIRSTNAME, new MaxLenghtConstraint(100, Person.FIRSTNAME, false, false));

        ConstraintManager.addConstraint(Collection.TITLE, new MaxLenghtConstraint(150, Collection.TITLE, false, false));
        ConstraintManager.addConstraint(Collection.PASSWORD, new MaxLenghtConstraint(150, Collection.PASSWORD, true, false));

        ConstraintManager.addConstraint(Film.TITLE, new MaxLenghtConstraint(150, Film.TITLE, false, false));
        ConstraintManager.addConstraint(Film.YEAR, new MaxLenghtConstraint(4, Film.YEAR, false, true));
        ConstraintManager.addConstraint(Film.DURATION, new MaxLenghtConstraint(4, Film.DURATION, false, true));
        ConstraintManager.addConstraint(Film.RESUME, new MaxLenghtConstraint(2000, Film.RESUME, false, false));
        ConstraintManager.addConstraint(Film.COMMENT, new MaxLenghtConstraint(2000, Film.COMMENT, false, false));
        ConstraintManager.addConstraint(Film.IMAGE, new MaxLenghtConstraint(200, Film.IMAGE, false, false));
        ConstraintManager.addConstraint(Film.FILEPATH, new MaxLenghtConstraint(200, Film.FILEPATH, false, false));
        ConstraintManager.addConstraint(Film.KIND, new NotNullConstraint(Film.KIND));
        ConstraintManager.addConstraint(Film.TYPE, new NotNullConstraint(Film.TYPE));

        ConstraintManager.addConstraint("saga.name", new MaxLenghtConstraint(150, "saga.name", false, false));

        ConstraintManager.addConstraint("kind.name", new MaxLenghtConstraint(150, "kind.name", false, false));

        ConstraintManager.addConstraint("type.name", new MaxLenghtConstraint(150, "type.name", false, false));

        ConstraintManager.addConstraint(Publication.HOST, new MaxLenghtConstraint(100, Publication.HOST, false, false));
        ConstraintManager.addConstraint(Publication.USER, new MaxLenghtConstraint(100, Publication.USER, true, false));
        ConstraintManager.addConstraint(Publication.PASSWORD, new MaxLenghtConstraint(100, Publication.PASSWORD, true, false));
        ConstraintManager.addConstraint(Publication.PATH, new MaxLenghtConstraint(200, Publication.PATH, false, false));
        ConstraintManager.addConstraint(Publication.PORT, new MaxLenghtConstraint(12, Publication.PORT, false, true));
    }

    @UnPlug
    public void unplug() {
        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Unplug Films ModuleContainer Start");

        for (Sorter sorter : sorters) {
            SorterFactory.getInstance().removeSorter(sorter);
        }

        PrimaryUtils.unplug();
        
        for(ConfigTabComponent component : configTabComponents){
            Managers.getManager(IViewManager.class).removeConfigTabComponent(component);
        }

        for(TabComponent component : tabComponents){
            Managers.getManager(IViewManager.class).removeTabComponent(component);
        }
        
        for (ChoiceAction action : choiceActions) {
            ChoiceActionFactory.removeChoiceAction(action);
        }

        removeFeatures();

        //Backup/Restore
        Managers.getManager(IFileManager.class).unregisterBackupWriter(FileType.JTD, jtdWriter);
        Managers.getManager(IFileManager.class).unregisterBackupWriter(FileType.XML, xmlWriter);

        Managers.getManager(IFileManager.class).unregisterBackupReader(FileType.XML, xmlReader);
        Managers.getManager(IFileManager.class).unregisterBackupReader(FileType.JTD, jtdReader);
        Managers.getManager(IFileManager.class).unregisterBackupReader(FileType.V3, dbv3Reader);
        Managers.getManager(IFileManager.class).unregisterBackupReader(FileType.V4, dbv4Reader);

        DataTypeManager.unbindDataType(IFilmsService.DATA_TYPE);
        DataTypeManager.unbindDataType(IActorService.DATA_TYPE);
        DataTypeManager.unbindDataType(IRealizersService.DATA_TYPE);
        DataTypeManager.unbindDataType(ITypesService.DATA_TYPE);
        DataTypeManager.unbindDataType(IKindsService.DATA_TYPE);
        DataTypeManager.unbindDataType(ISagasService.DATA_TYPE);

        //I18n

        Managers.getManager(ILanguageManager.class).removeBasename(BASENAME);

        Managers.getManager(ISchemaManager.class).unregisterSchema(schema);

        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Unplug Films ModuleContainer End");
    }

    private void removeFeatures() {
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.HELP).removeSubFeature(sitesFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.FILE).removeSubFeature(refreshFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.FILE).removeSubFeature(printFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.FILE).removeSubFeature(importFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.FILE).removeSubFeature(exportFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.FILE).removeSubFeature(importFolderFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.EDIT).removeSubFeature(searchFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.EDIT).removeSubFeature(publicationFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.EDIT).removeSubFeature(coverFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.ADVANCED).removeSubFeature(videoFeature);
        Managers.getManager(IFeatureManager.class).removeFeature(realizersFeature);
        Managers.getManager(IFeatureManager.class).removeFeature(actorsFeature);
        Managers.getManager(IFeatureManager.class).removeFeature(filmsFeature);
        Managers.getManager(IFeatureManager.class).removeFeature(othersFeature);
        Managers.getManager(IFeatureManager.class).removeFeature(lendingsFeature);
    }

    @Override
    public Configuration getConfiguration() {
        return config;
    }

    @Override
    public Feature getFilmsFeature() {
        return filmsFeature;
    }

    @Override
    public Feature getActorsFeature() {
        return actorsFeature;
    }

    @Override
    public Feature getRealizersFeature() {
        return realizersFeature;
    }

    public void setConfigTabComponents(ConfigTabComponent[] configTabComponents) {
        this.configTabComponents = configTabComponents;
    }

    public void setTabComponents(TabComponent[] tabComponents) {
        this.tabComponents = tabComponents;
    }

    public void setSorters(Sorter[] sorters) {
        this.sorters = sorters;
    }

    public void setChoiceActions(ChoiceAction[] choiceActions) {
        this.choiceActions = choiceActions;
    }
}