package org.jtheque.films.persistence;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.schema.Schema;
import org.jtheque.core.managers.schema.Insert;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.utils.bean.Version;
import org.jtheque.films.persistence.dao.able.IDaoActors;
import org.jtheque.films.persistence.dao.able.IDaoFilms;
import org.jtheque.primary.dao.able.IDaoCollections;
import org.jtheque.films.persistence.dao.able.IDaoRealizers;
import org.jtheque.primary.dao.able.*;
import org.jtheque.primary.PrimaryUtils;

import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;

/**
 * The database schema for the Movies Module.
 *
 * @author Baptiste Wicht
 */
public final class FilmsSchema implements Schema {
    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new MoviesSchema.
     */
    public FilmsSchema(){
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public Version getVersion() {
        return new Version("1.1");
    }

    @Override
    public String getName() {
        return "Films-Schema";
    }

    @Override
    public void install() {
        createDataTable();
        createReferentialIntegrityConstraints();
    }

    @Override
    public void update(Version from) {
        if("1.0".equals(from.getVersion())){
            createReferentialIntegrityConstraints();            
        }
    }

    private void createDataTable() {
        jdbcTemplate.update("CREATE TABLE " + IDaoActors.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(100) NOT NULL, FIRSTNAME VARCHAR(100) NOT NULL, NOTE INT, THE_COUNTRY_FK INT, CONSTRAINT UNIQUE_ACTOR UNIQUE(NAME, FIRSTNAME))");
        jdbcTemplate.update("CREATE TABLE " + IDaoRealizers.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(100) NOT NULL, FIRSTNAME VARCHAR(100) NOT NULL, NOTE INT, THE_COUNTRY_FK INT, CONSTRAINT UNIQUE_REALIZER UNIQUE(NAME, FIRSTNAME))");
        jdbcTemplate.update("CREATE TABLE " + IDaoFilms.TABLE + " (ID INT IDENTITY PRIMARY KEY, TITLE VARCHAR(150) NOT NULL UNIQUE, YEAR INT, COMMENT VARCHAR(2000),DURATION INT,IMAGE VARCHAR(200),NOTE INT, FILEPATH VARCHAR(250), RESUME VARCHAR(2000),THE_REALIZER_FK INT,THE_COLLECTION_FK INT NOT NULL, THE_KIND_FK INT, THE_LANGUAGE_FK INT, THE_LENDING_FK INT, THE_SAGA_FK INT, THE_TYPE_FK INT)");
        jdbcTemplate.update("CREATE TABLE " + IDaoFilms.ACTORS_FILMS_TABLE + " (THE_FILM_FK INT NOT NULL, THE_ACTOR_FK INT NOT NULL)");
        jdbcTemplate.update("CREATE TABLE " + IDaoFilms.KINDS_FILMS_TABLE + " (THE_FILM_FK INT NOT NULL, THE_KIND_FK INT NOT NULL)");

        jdbcTemplate.update("CREATE INDEX FILM_ACTORS_IDX ON " + IDaoActors.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX FILM_REALIZERS_IDX ON " + IDaoRealizers.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX FILM_IDX ON " + IDaoFilms.TABLE + "(ID)");
    }

    private void createReferentialIntegrityConstraints() {
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.ACTORS_FILMS_TABLE + " ADD FOREIGN KEY (THE_FILM_FK) REFERENCES  " + IDaoFilms.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.ACTORS_FILMS_TABLE + " ADD FOREIGN KEY (THE_ACTOR_FK) REFERENCES  " + IDaoActors.TABLE + "  (ID) ON UPDATE SET NULL");
        
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.KINDS_FILMS_TABLE + " ADD FOREIGN KEY (THE_FILM_FK) REFERENCES  " + IDaoFilms.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.KINDS_FILMS_TABLE + " ADD FOREIGN KEY (THE_KIND_FK) REFERENCES  " + IDaoKinds.TABLE + "  (ID) ON UPDATE SET NULL");
        
        jdbcTemplate.update("ALTER TABLE " + IDaoActors.TABLE + " ADD FOREIGN KEY (THE_COUNTRY_FK) REFERENCES  " + IDaoCountries.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoRealizers.TABLE + " ADD FOREIGN KEY (THE_COUNTRY_FK) REFERENCES  " + IDaoCountries.TABLE + "  (ID) ON UPDATE SET NULL");
        
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.TABLE + " ADD FOREIGN KEY (THE_KIND_FK) REFERENCES  " + IDaoKinds.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.TABLE + " ADD FOREIGN KEY (THE_REALIZER_FK) REFERENCES  " + IDaoRealizers.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.TABLE + " ADD FOREIGN KEY (THE_COLLECTION_FK) REFERENCES  " + IDaoCollections.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.TABLE + " ADD FOREIGN KEY (THE_LANGUAGE_FK) REFERENCES  " + IDaoLanguages.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.TABLE + " ADD FOREIGN KEY (THE_LENDING_FK) REFERENCES  " + IDaoLendings.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.TABLE + " ADD FOREIGN KEY (THE_SAGA_FK) REFERENCES  " + IDaoSagas.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoFilms.TABLE + " ADD FOREIGN KEY (THE_TYPE_FK) REFERENCES  " + IDaoTypes.TABLE + "  (ID) ON UPDATE SET NULL");
    }
    
    @Override
    public void importDataFromHSQL(Iterable<Insert> inserts) {
        for(Insert insert : inserts){
            if("OD_SAGA_FILM".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoSagas.TABLE + " (ID, NAME, IMPL) VALUES (?,?,?)", 
                        insert.getInt(0), insert.getString(2), "Films");
            } else if("OD_KIND".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoKinds.TABLE + " (ID, NAME, IMPL) VALUES (?,?,?)", 
                        insert.getInt(0), insert.getString(2), "Films");
            } else if("OD_TYPE".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoTypes.TABLE + " (ID, NAME, IMPL) VALUES (?,?,?)", 
                        insert.getInt(0), insert.getString(2), "Films");
            } else if("FILM_ACTOR".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoFilms.ACTORS_FILMS_TABLE + " (THE_FILM_FK, THE_ACTOR_FK) VALUES (?,?)", 
                        insert.getInt(0), insert.getInt(1));
            } else if("FILM_KIND".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoFilms.KINDS_FILMS_TABLE + " (THE_FILM_FK, THE_KIND_FK) VALUES (?,?)", 
                        insert.getInt(0), insert.getInt(1));
            } else if("OD_LENDING".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoLendings.TABLE + " (ID, DATE, THE_BORROWER_FK, IMPL) VALUES (?,?,?,?)", 
                        insert.getInt(0), insert.getInt(2), insert.getInt(3), "Films");
            } else if("OD_ACTOR".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoActors.TABLE + " (ID, NAME, FIRSTNAME, NOTE, THE_COUNTRY_FK) VALUES (?,?,?,?,?)",
                        insert.getInt(0), insert.getString(3), insert.getString(2), insert.getInt(4), insert.getInt(5));
            } else if("OD_REALIZER".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoRealizers.TABLE + " (ID, NAME, FIRSTNAME, NOTE, THE_COUNTRY_FK) VALUES (?,?,?,?,?)",
                        insert.getInt(0), insert.getString(3), insert.getString(2), insert.getInt(4), insert.getInt(5));
            } else if("OD_FILM_COLLECTION".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoCollections.TABLE + " (ID, TITLE, PROTECTED, PASSWORD, IMPL) VALUES(?,?,?,?,?)",
                        insert.getInt(0), insert.getString(4), insert.getBoolean(3), insert.getString(2), PrimaryUtils.getPrimaryImpl());
            } else if("OD_FILM".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoFilms.TABLE + " (ID, TITLE, YEAR, COMMENT,DURATION,IMAGE,NOTE,RESUME,THE_REALIZER_FK,THE_COLLECTION_FK, THE_KIND_FK, THE_LANGUAGE_FK, THE_LENDING_FK, THE_SAGA_FK, THE_TYPE_FK, FILEPATH) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
                        insert.getInt(0), insert.getString(7), insert.getInt(8), insert.getString(2), insert.getInt(3), insert.getString(4), insert.getInt(5), insert.getString(6),
                        insert.getInt(13), insert.getInt(9), insert.getInt(10), insert.getInt(11), insert.getInt(12), insert.getInt(15), insert.getInt(14), insert.getString(16));
            }
        }
    }
}