package org.jtheque.films.persistence.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.films.persistence.dao.able.IDaoActors;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.primary.dao.able.IDaoCountries;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * Cette classe permet de grer la persistance des acteurs.
 *
 * @author Baptiste Wicht
 */
public final class DaoActors extends GenericDao<ActorImpl> implements IDaoActors {
    private final ParameterizedRowMapper<ActorImpl> rowMapper = new ActorRowMapper();
    private final QueryMapper queryMapper = new ActorQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private IDaoCountries daoCountries;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new DaoActors. 
     *
     */
    public DaoActors() {
        super(TABLE);
    }

    @Override
    public List<ActorImpl> getActors() {
        return getAll();
    }

    @Override
    public ActorImpl getActor(String firstName, String name) {
        ActorImpl actor = jdbcTemplate.queryForObject("SELECT * FROM " + TABLE + " WHERE NAME = ? AND FIRSTNAME = ?", rowMapper, name, firstName);

        if (isNotInCache(actor.getId())) {
            getCache().put(actor.getId(), actor);
        }

        return getCache().get(actor.getId());
    }

    @Override
    public boolean exists(String firstName, String name) {
        return getActor(firstName, name) != null;
    }

    @Override
    public boolean exist(ActorImpl actor) {
        return getActor(actor.getFirstName(), actor.getName()) != null;
    }

    @Override
    public ActorImpl getActor(int id) {
        return get(id);
    }

    @Override
    protected ParameterizedRowMapper<ActorImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<ActorImpl> actors = persistenceContext.getSortedList(TABLE, rowMapper);

        for (ActorImpl actor : actors) {
            getCache().put(actor.getId(), actor);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        ActorImpl actor = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, actor);
    }

    /**
     * A row mapper to map resultset to actor.
     *
     * @author Baptiste Wicht
     */
    private final class ActorRowMapper implements ParameterizedRowMapper<ActorImpl> {
        @Override
        public ActorImpl mapRow(ResultSet rs, int i) throws SQLException {
            ActorImpl actor = new ActorImpl();

            actor.setId(rs.getInt("ID"));
            actor.setName(rs.getString("NAME"));
            actor.setFirstName(rs.getString("FIRSTNAME"));
            actor.setTheCountry(daoCountries.getCountry(rs.getInt("THE_COUNTRY_FK")));
            actor.setNote(DaoNotes.getINSTANCE().getNote(NoteType.getEnum(rs.getInt("NOTE"))));

            return actor;
        }
    }

    /**
     * A query mapper to map actor to sql query. 
     *
     * @author Baptiste Wicht
     */
    private static final class ActorQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            ActorImpl actor = (ActorImpl) entity;

            String query = "INSERT INTO " + TABLE + " (NAME, FIRSTNAME, THE_COUNTRY_FK, NOTE) VALUES(?,?,?,?)";

            Object[] parameters = {
                    actor.getName(),
                    actor.getFirstName(),
                    actor.getTheCountry().getId(),
                    actor.getNote().getValue().intValue()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            ActorImpl actor = (ActorImpl) entity;

            String query = "UPDATE " + TABLE + " SET NAME = ?, FIRSTNAME = ?, THE_COUNTRY_FK = ?, NOTE = ? WHERE ID = ?";

            Object[] parameters = {
                    actor.getName(),
                    actor.getFirstName(),
                    actor.getTheCountry().getId(),
                    actor.getNote().getValue().intValue(),
                    actor.getId()};

            return new Query(query, parameters);
        }
    }
}