package org.jtheque.films.persistence.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.films.persistence.dao.able.IDaoRealizers;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.primary.dao.able.IDaoCountries;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * Cette classe permet de grer la persistance des ralisateurs.
 *
 * @author Baptiste Wicht
 */
public final class DaoRealizers extends GenericDao<RealizerImpl> implements IDaoRealizers {
    private final ParameterizedRowMapper<RealizerImpl> rowMapper = new RealizerRowMapper();
    private final QueryMapper queryMapper = new RealizerQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private IDaoCountries daoCountries;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new DaoRealizers.
     */
    public DaoRealizers() {
        super(TABLE);
    }

    @Override
    public List<RealizerImpl> getRealizers() {
        return getAll();
    }

    @Override
    public boolean exist(RealizerImpl realizer) {
        return getRealizer(realizer.getFirstName(), realizer.getName()) != null;
    }

    @Override
    public RealizerImpl getRealizer(int id) {
        return get(id);
    }

    @Override
    public RealizerImpl getRealizer(String firstName, String name) {
        List<RealizerImpl> realizers = jdbcTemplate.query("SELECT * FROM " + TABLE + " WHERE NAME = ? AND FIRSTNAME = ?", rowMapper, name, firstName);
        
        if(realizers.isEmpty()){
            return null;
        }
        
        RealizerImpl realizer = realizers.get(0);
        
        if (isNotInCache(realizer.getId())) {
            getCache().put(realizer.getId(), realizer);
        }

        return getCache().get(realizer.getId());
    }

    @Override
    public boolean exists(String name, String lastName) {
        return getRealizer(name, lastName) != null;
    }

    @Override
    protected ParameterizedRowMapper<RealizerImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<RealizerImpl> realizers = persistenceContext.getSortedList(TABLE, rowMapper);

        for (RealizerImpl realizer : realizers) {
            getCache().put(realizer.getId(), realizer);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        RealizerImpl realizer = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, realizer);
    }

    /**
     * A row mapper to map resultset to realizer.
     *
     * @author Baptiste Wicht
     */
    private final class RealizerRowMapper implements ParameterizedRowMapper<RealizerImpl> {
        @Override
        public RealizerImpl mapRow(ResultSet rs, int i) throws SQLException {
            RealizerImpl realizer = new RealizerImpl();

            realizer.setId(rs.getInt("ID"));
            realizer.setName(rs.getString("NAME"));
            realizer.setFirstName(rs.getString("FIRSTNAME"));
            realizer.setTheCountry(daoCountries.getCountry(rs.getInt("THE_COUNTRY_FK")));
            realizer.setNote(DaoNotes.getINSTANCE().getNote(NoteType.getEnum(rs.getInt("NOTE"))));

            return realizer;
        }
    }

    /**
     * A query mapper to map realizer to sql query.
     *
     * @author Baptiste Wicht
     */
    private static final class RealizerQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            RealizerImpl realizer = (RealizerImpl) entity;

            String query = "INSERT INTO " + TABLE + " (NAME, FIRSTNAME, THE_COUNTRY_FK, NOTE) VALUES(?,?,?,?)";

            Object[] parameters = {
                    realizer.getName(),
                    realizer.getFirstName(),
                    realizer.getTheCountry().getId(),
                    realizer.getNote().getValue().intValue()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            RealizerImpl realizer = (RealizerImpl) entity;

            String query = "UPDATE " + TABLE + " SET NAME = ?, FIRSTNAME = ?, THE_COUNTRY_FK = ?, NOTE = ? WHERE ID = ?";

            Object[] parameters = {
                    realizer.getName(),
                    realizer.getFirstName(),
                    realizer.getTheCountry().getId(),
                    realizer.getNote().getValue().intValue(),
                    realizer.getId()};

            return new Query(query, parameters);
        }
    }
}