package org.jtheque.films.services.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.persistence.able.DataListener;
import org.jtheque.utils.StringUtils;
import org.jtheque.utils.print.PrintUtils;
import org.jtheque.films.persistence.dao.able.IDaoFilms;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.abstraction.Film;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.INotesService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.services.impl.utils.VideoFile;
import org.jtheque.primary.services.able.ILanguagesService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.primary.services.able.ITypesService;
import org.jtheque.primary.od.KindImpl;

import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * The implementation of the films service.
 *
 * @author Baptiste Wicht
 */
public final class FilmsService implements IFilmsService {
    @Resource
    private IDaoFilms daoFilms;

    @Resource
    private INotesService notesService;

    @Resource
    private IRealizersService realizersService;

    @Resource
    private IKindsService kindsService;

    @Resource
    private ITypesService typesService;

    @Resource
    private ILanguagesService languagesService;

    @Override
    @Transactional
    public void create(FilmImpl film) {
        daoFilms.create(film);
    }

    @Override
    @Transactional
    public void save(FilmImpl film) {
        daoFilms.save(film);
    }

    @Override
    @Transactional
    public boolean delete(FilmImpl film) {
        return daoFilms.delete(film);
    }

    @Override
    @Transactional
    public void createAll(List<FilmImpl> films) {
        daoFilms.createAll(films);
    }

    @Override
    public boolean isNoFilms() {
        return daoFilms.getFilms().size() <= 0;
    }

    @Override
    public List<FilmImpl> getFilms() {
        return daoFilms.getFilms();
    }

    @Override
    public List<FilmImpl> getAllFilms() {
        return daoFilms.getAllFilms();
    }

    @Override
    public void addDataListener(DataListener listener) {
        daoFilms.addDataListener(listener);
    }

    @Override
    public List<FilmImpl> getDatas() {
        return daoFilms.getFilms();
    }

    @Override
    public FilmImpl getEmptyFilm() {
        FilmImpl emptyFilm = new FilmImpl();

        emptyFilm.setTitle(Managers.getManager(ILanguageManager.class).getMessage("generic.view.actions.new"));
        emptyFilm.setNote(notesService.getDefaultNote());
        emptyFilm.setTheRealizer(realizersService.getDefaultRealizer());
        emptyFilm.getKinds().add(kindsService.getDefaultKind());
        emptyFilm.setTheType(typesService.getDefaultType());
        emptyFilm.setTheLanguage(languagesService.getDefaultLanguage());

        return emptyFilm;
    }

    @Override
    public void printListOfFilms() {
        List<String> list = new ArrayList<String>(25);

        list.add("Liste des films");

        for (FilmImpl f : daoFilms.getFilms()) {
            list.add(f.getAffichableText());
        }

        PrintUtils.printArrayString(list);
    }

    @Override
    public String generateEmail(FilmImpl film) {
        return generateFilmDescription(film);
    }

    /**
     * Create a string description for a film.
     *
     * @param film The film to generate the description for.
     * @return The string description of the film.
     */
    private static String generateFilmDescription(Film film) {
        StringBuilder builder = new StringBuilder(250);

        builder.append("Salut, \n\n je t'envoie la fiche d'un film qui pourrait t'intresser : \n");
        builder.append("Titre : ");
        builder.append(film.getTitle());
        builder.append("\nGenre : ");

        for (KindImpl kind : film.getKinds()) {
            builder.append("\n - ").append(kind.getAffichableText());
        }

        builder.append("\nRalisateur : ");
        builder.append(film.getTheRealizer());
        builder.append("\nAnne : ");
        builder.append(film.getYear());
        builder.append("\nActeurs : ");

        for (ActorImpl actor : film.getActors()) {
            builder.append("\n - ").append(actor.getAffichableText());
        }

        return builder.toString();
    }

    @Override
    public String generateFilmDescriptionForPrinting(FilmImpl film) {
        return generateFilmDescription(film);
    }

    @Override
    public List<VideoFile> getVideoFiles() {
        List<VideoFile> videoFiles = new ArrayList<VideoFile>(10);

        for (FilmImpl film : daoFilms.getFilms()) {
            if (!StringUtils.isEmpty(film.getFilePath())) {
                String path;

                if (film.getFilePath().contains("/")) {
                    path = film.getFilePath().substring(film.getFilePath().lastIndexOf('/') + 1);
                } else if (film.getFilePath().contains("\\")) {
                    path = film.getFilePath().substring(film.getFilePath().lastIndexOf('\\') + 1);
                } else {
                    path = film.getFilePath();
                }

                videoFiles.add(new VideoFile(path, film));
            }
        }

        return videoFiles;
    }

    @Override
    @Transactional
    public void clearAll() {
        daoFilms.clearAll();
    }

    @Override
    public String getDataType() {
        return DATA_TYPE;
    }
}