package org.jtheque.films.services.impl.utils.file.imports;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.services.impl.utils.DataUtils;
import org.jtheque.films.services.impl.utils.file.jt.JTFFile;
import org.jtheque.films.services.impl.utils.file.jt.reader.JTFFileReader;
import org.jtheque.films.utils.Constants;
import org.jtheque.films.utils.Constants.Files.FileType;
import org.jtheque.primary.od.CountryImpl;
import org.jtheque.primary.od.KindImpl;
import org.jtheque.primary.services.able.ICountriesService;
import org.jtheque.primary.services.able.ILanguagesService;
import org.jtheque.primary.services.able.ITypesService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.utils.io.FileException;

import javax.annotation.Resource;

/**
 * Cette classe permet d'importer des fichiers JTF dans la base de donnes.
 *
 * @author Baptiste Wicht
 */
public final class JTFImporter extends AbstractImporter {
    private final ILanguageManager resources = Managers.getManager(ILanguageManager.class);

    @Resource
    private ICountriesService countriesService;

    @Resource
    private ITypesService typesService;

    @Resource
    private ILanguagesService languagesService;

    @Resource
    private IFilmsService filmsService;

    @Resource
    private IActorService actorService;

    @Resource
    private IRealizersService realizersService;

    private final DaoNotes daoNotes = DaoNotes.getINSTANCE();

    @Resource
    private IKindsService kindsService;

    /**
     * Construct a new JTFImporter.
     */
    public JTFImporter() {
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public boolean canImportFrom(FileType fileType) {
        return fileType == FileType.JTF;
    }

    /**
     * Import from a JTFFile.
     *
     * @param file The JTF File.
     */
    public void importFrom(JTFFile file) throws FileException {
        if (file.getHeader().getFileVersion() != Constants.Files.Versions.JTF.FIRST ||
                file.getHeader().getFileVersion() != Constants.Files.Versions.JTF.SECOND) {
            throw new FileException(resources.getMessage("errors.file.unsupportedversion"));
        } else if (file.isValid()) {
            if (file.getLangue() != null) {
                if (languagesService.exist(file.getLangue())) {
                    file.setLangue(languagesService.getLanguage(file.getLangue().getName()));
                } else {
                    languagesService.create(file.getLangue());
                }

                file.getFilm().setTheLanguage(file.getLangue());
            }

            for (CountryImpl country : file.getCountries()) {
                if (countriesService.exist(country)) {
                    country.setId(countriesService.getCountry(country.getName()).getId());
                } else {
                    countriesService.create(country);
                }
            }

            for (KindImpl kind : file.getKinds()) {
                if (kindsService.exists(kind)) {
                    kind.setId(kindsService.getKind(kind.getName()).getId());
                } else {
                    kindsService.create(kind);
                }
            }

            file.getFilm().setKinds(file.getKinds());

            if (file.getType() != null) {
                if (typesService.exists(file.getType())) {
                    file.getType().setId(typesService.getType(file.getType().getName()).getId());
                } else {
                    typesService.create(file.getType());
                }

                file.getFilm().setTheType(file.getType());
            }

            if (file.getRealizer() != null) {
                file.getRealizer().setNote(daoNotes.getNote(NoteType.getEnum(file.getRealizer().getTemporaryContext().getIntNote())));
                file.getRealizer().setTheCountry(DataUtils.getDataByTemporaryId(
                        file.getRealizer().getTemporaryContext().getCountry(), file.getCountries()));

                if (realizersService.exists(file.getRealizer())) {
                    file.getRealizer().setId(realizersService.getRealizer(file.getRealizer().getFirstName(), file.getRealizer().getName()).getId());
                } else {
                    realizersService.create(file.getRealizer());
                }

                file.getFilm().setTheRealizer(file.getRealizer());
            }

            for (ActorImpl actor : file.getActeurs()) {
                actor.setNote(daoNotes.getNote(NoteType.getEnum(actor.getTemporaryContext().getIntNote())));
                actor.setTheCountry(DataUtils.getDataByTemporaryId(
                        actor.getTemporaryContext().getCountry(), file.getCountries()));

                if (actorService.exist(actor)) {
                    actor.setId(actorService.getActor(actor.getFirstName(), actor.getName()).getId());
                } else {
                    actorService.create(actor);
                }
            }

            file.getFilm().setNote(daoNotes.getNote(NoteType.getEnum(file.getFilm().getTemporaryContext().getIntNote())));
            file.getFilm().setActors(file.getActeurs());

            filmsService.create(file.getFilm());
        } else {
            throw new FileException(resources.getMessage("errors.file.structureerror"));
        }
    }

    @Override
    public void importFrom(String filePath) throws FileException {
        JTFFileReader reader = new JTFFileReader();

        importFrom((JTFFile) reader.readFile(filePath));
    }
}