package org.jtheque.films.services.impl.utils.file.restore;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.file.IFileManager;
import org.jtheque.core.managers.file.able.BackupReader;
import org.jtheque.utils.bean.IntDate;
import org.jtheque.core.utils.file.jt.impl.JTDFile;
import org.jtheque.core.utils.file.jt.able.JTNotZippedFile;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.impl.utils.file.imports.ImporterUtils;
import org.jtheque.primary.od.*;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A reader for JTD Backup.
 *
 * @author Baptiste Wicht
 */
public final class JTDBackupReader implements BackupReader {
    private List<FilmImpl> films;
    private List<ActorImpl> actors;
    private List<RealizerImpl> realizers;
    private List<KindImpl> kinds;
    private List<TypeImpl> types;
    private List<LanguageImpl> languages;
    private List<CountryImpl> countries;
    private List<BorrowerImpl> borrowers;
    private List<LendingImpl> lendings;

    @Override
    public void readBackup(Object object) {
        JTDFile file = (JTDFile) object;
        DataInputStream stream = file.getStream();

        try {
            readFilms(stream, file);
            readActors(stream, file);
            readRealizers(stream, file);
            readLanguages(stream, file);
            readKinds(stream, file);
            readTypes(stream, file);
            readCountries(stream, file);
            readLendings(stream, file);
            readBorrowers(stream, file);
        } catch (IOException e) {
            Managers.getManager(ILoggingManager.class).getLogger(getClass()).exception(e);
        }
    }

    @Override
    public void persistTheData() {
        ImporterUtils.persistDataOfImport(films, actors,
                realizers, kinds, types,
                languages, countries,
                borrowers, lendings);
    }

    /**
     * Read the borrowers.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readBorrowers(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des emprunteurs
        if (stream.readInt() == IFileManager.CONTENT) {
            borrowers = new ArrayList<BorrowerImpl>(10);

            boolean endOfBorrowersList = false;
            while (!endOfBorrowersList) {
                BorrowerImpl borrower = new BorrowerImpl();

                borrower.getTemporaryContext().setId(stream.readInt());
                borrower.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                borrower.setFirstName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                borrower.setEmail(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                borrowers.add(borrower);

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfBorrowersList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the lendings.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readLendings(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des emprunts
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfLendingsList = false;
            lendings = new ArrayList<LendingImpl>(10);

            while (!endOfLendingsList) {
                LendingImpl lending = new LendingImpl();

                lending.getTemporaryContext().setId(stream.readInt());
                lending.setDate(new IntDate(stream.readInt()));
                lending.getTemporaryContext().setBorrower(stream.readInt());
                lending.getTemporaryContext().setFilm(stream.readInt());

                lendings.add(lending);

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfLendingsList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the countries.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readCountries(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des pays
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfCountries = false;
            countries = new ArrayList<CountryImpl>(20);

            while (!endOfCountries) {
                CountryImpl country = new CountryImpl();

                country.getTemporaryContext().setId(stream.readInt());
                country.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                countries.add(country);

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfCountries = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the types.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readTypes(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des types
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfGenressList = false;
            types = new ArrayList<TypeImpl>(10);

            while (!endOfGenressList) {
                TypeImpl type = new TypeImpl();

                type.getTemporaryContext().setId(stream.readInt());
                type.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                types.add(type);

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfGenressList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the kinds.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readKinds(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des genres
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfGenressList = false;
            kinds = new ArrayList<KindImpl>(10);

            while (!endOfGenressList) {
                KindImpl genre = new KindImpl();

                genre.getTemporaryContext().setId(stream.readInt());
                genre.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                kinds.add(genre);

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfGenressList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the languages.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readLanguages(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des langues
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfLanguageesList = false;
            languages = new ArrayList<LanguageImpl>(10);

            while (!endOfLanguageesList) {
                LanguageImpl language = new LanguageImpl();

                language.getTemporaryContext().setId(stream.readInt());
                language.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                languages.add(language);

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfLanguageesList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the realizers.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readRealizers(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des ralisateurs
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfRealisateursList = false;
            realizers = new ArrayList<RealizerImpl>(20);

            while (!endOfRealisateursList) {
                RealizerImpl realizer = new RealizerImpl();

                realizer.getTemporaryContext().setId(stream.readInt());
                realizer.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                realizer.setFirstName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                realizer.getTemporaryContext().setCountry(stream.readInt());
                realizer.getTemporaryContext().setIntNote(stream.readInt());

                realizers.add(realizer);

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfRealisateursList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the actors.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readActors(DataInputStream stream, JTNotZippedFile file) throws IOException {
        //Si y a des acteurs
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfActeursList = false;
            actors = new ArrayList<ActorImpl>(50);

            while (!endOfActeursList) {
                ActorImpl acteur = new ActorImpl();

                acteur.getTemporaryContext().setId(stream.readInt());
                acteur.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                acteur.setFirstName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                acteur.getTemporaryContext().setCountry(stream.readInt());
                acteur.getTemporaryContext().setIntNote(stream.readInt());

                actors.add(acteur);

                long separator = stream.readLong();
                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfActeursList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }

    /**
     * Read the films.
     *
     * @param stream The input stream.
     * @param file   The file.
     * @throws IOException Thrown when an error occurs during the reading process.
     */
    private void readFilms(DataInputStream stream, JTNotZippedFile file) throws IOException {
        if (stream.readInt() == IFileManager.CONTENT) {
            boolean endOfFilmsList = false;
            films = new ArrayList<FilmImpl>(25);

            while (!endOfFilmsList) {
                FilmImpl film = new FilmImpl();

                film.getTemporaryContext().setId(stream.readInt());
                film.setTitle(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                //Si y a des genres
                if (stream.readInt() == IFileManager.CONTENT) {
                    boolean endOfKindsList = false;
                    List<Integer> kindsFilm = new ArrayList<Integer>(12);

                    while (!endOfKindsList) {
                        kindsFilm.add(stream.readInt());

                        long separator = stream.readLong();

                        if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                            endOfKindsList = true;
                        } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                            file.setCorrectSeparators(false);
                        }
                    }

                    film.getTemporaryContext().setKinds(kindsFilm);
                } else {
                    film.getTemporaryContext().setKinds(Collections.<Integer>emptyList());
                }

                film.getTemporaryContext().setType(stream.readInt());
                film.getTemporaryContext().setRealizer(stream.readInt());
                film.getTemporaryContext().setLanguage(stream.readInt());
                film.setYear(stream.readInt());
                film.setDuration(stream.readInt());
                film.getTemporaryContext().setLending(stream.readInt());
                film.setResume(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                film.setComment(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                //Si y a des acteurs
                if (stream.readInt() == IFileManager.CONTENT) {
                    boolean endOfActeursList = false;
                    List<Integer> acteurs = new ArrayList<Integer>(25);

                    while (!endOfActeursList) {
                        acteurs.add(stream.readInt());

                        long separator = stream.readLong();

                        if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                            endOfActeursList = true;
                        } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                            file.setCorrectSeparators(false);
                        }
                    }

                    film.getTemporaryContext().setActors(acteurs);
                } else {
                    film.getTemporaryContext().setActors(new ArrayList<Integer>(0));
                }

                long separator = stream.readLong();

                if (separator == IFileManager.JTCATEGORYSEPARATOR) {
                    endOfFilmsList = true;
                } else if (separator != IFileManager.JTINTERNSEPARATOR) {
                    file.setCorrectSeparators(false);
                }

                films.add(film);
            }
        } else if (stream.readLong() != IFileManager.JTCATEGORYSEPARATOR) {
            file.setCorrectSeparators(false);
        }
    }
}