package org.jtheque.films.services.impl.utils.web.analyzers;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.utils.StringUtils;
import org.jtheque.films.IFilmsModule;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.INotesService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.utils.Constants.Site;
import org.jtheque.primary.services.able.ICountriesService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.primary.utils.web.analyzers.generic.field.FieldGetter;
import org.jtheque.primary.utils.web.analyzers.generic.GenericGenerator;
import org.jtheque.primary.utils.web.analyzers.generic.operation.ScannerPossessor;
import org.jtheque.primary.od.KindImpl;

import javax.annotation.Resource;
import java.util.HashSet;
import java.util.Set;
import java.util.Scanner;

/**
 * A generic film analyzer. It seems an analyzer who takes its parser information from an XML file.
 *
 * @author Baptiste Wicht
 */
public final class GenericFilmAnalyzer extends AbstractFilmAnalyzer implements ScannerPossessor {
    @Resource
    private IKindsService kindsService;

    @Resource
    private IRealizersService realizersService;

    @Resource
    private INotesService notesService;

    @Resource
    private ICountriesService countriesService;

    @Resource
    private IActorService actorService;

    @Resource
    private IFilmsModule filmsModule;

    private final Site site;

    /**
     * The generator of the field getters.
     */
    private final GenericGenerator generator;

    private FieldGetter dateGetter;
    private FieldGetter durationGetter;
    private FieldGetter imageGetter;
    private FieldGetter kindGetter;
    private FieldGetter realizerGetter;
    private FieldGetter resumeGetter;
    private FieldGetter actorsGetter;

    /**
     * Construct a new GenericFilmAnalyzer for a specific site from a file.
     *
     * @param file The XML file with the informations of the parse.
     * @param site The site for which this parser is done.
     */
    public GenericFilmAnalyzer(String file, Site site) {
        super();

        generator = new GenericGenerator(this);
        generator.generate("org/jtheque/films/ressources/analyzers/" + file);

        init();

        Managers.getManager(IBeansManager.class).inject(this);

        this.site = site;
    }

    /**
     * Construct a new GenericFilmAnalyzer.
     *
     * @param generator The generator to use.
     * @param site The site. 
     */
    public GenericFilmAnalyzer(GenericGenerator generator, Site site) {
        super();

        this.generator = generator;

        init();

        Managers.getManager(IBeansManager.class).inject(this);

        this.site = site;
    }

    @Override
    public Scanner getScanner() {
        return null;
    }

    @Override
    public boolean canAnalyze(Site site) {
        return this.site == site;
    }

    /**
     * Init the parser.
     */
    private void init() {
        dateGetter = generator.getFieldGetter("date");
        durationGetter = generator.getFieldGetter("duration");
        imageGetter = generator.getFieldGetter("image");
        kindGetter = generator.getFieldGetter("kind");
        realizerGetter = generator.getFieldGetter("realizer");
        resumeGetter = generator.getFieldGetter("resume");
        actorsGetter = generator.getFieldGetter("actors");
    }

    @Override
    public void findDate(String line) {
        if (isDateDo()) {
            return;
        }

        if (dateGetter.mustGet(line)) {
            line = dateGetter.performOperations(line, this);

            String value = dateGetter.getValue(line);

            if (value != null) {
                getFilm().setYear(Integer.parseInt(value));

                setDate(true);
            }
        }
    }

    @Override
    public void findDuration(String line) {
        if (isDurationDo()) {
            return;
        }

        if (durationGetter.mustGet(line)) {
            line = durationGetter.performOperations(line, this);

            String value = durationGetter.getValue(line);

            if (value != null) {
                int minutes;

                if (value.contains("h")) {
                    String[] heure = value.split("h");
                    minutes = Integer.parseInt(heure[0]) * 60 + Integer.parseInt(heure[1]);
                } else {
                    minutes = Integer.parseInt(value);
                }

                getFilm().setDuration(minutes);

                setDuration(true);
            }
        }
    }

    @Override
    public void findImage(String line) {
        if (isImageDo()) {
            return;
        }

        if (imageGetter.mustGet(line)) {
            line = imageGetter.performOperations(line, this);

            String value = imageGetter.getValue(line);

            if (value != null) {
                AnalyzerUtils.downloadMiniature(getFilm(), value);

                setImage(true);
            }
        }
    }

    @Override
    public void findKind(String line) {
        if (isKindDo()) {
            return;
        }

        if (kindGetter.mustGet(line)) {
            line = kindGetter.performOperations(line, this);

            String value = kindGetter.getValue(line);

            if (StringUtils.isNotEmpty(value)) {
                value = StringUtils.setFirstLetterUpper(value);

                if (kindsService.exists(value)) {
                    getFilm().getKinds().add(kindsService.getKind(value));
                } else {
                    KindImpl kind = new KindImpl();

                    kind.setName(value);

                    kindsService.create(kind);

                    getFilm().getKinds().add(kind);
                }

                setKind(true);
            }
        }
    }


    @Override
    public void findRealizer(String line) {
        if (isRealizerDo()) {
            return;
        }

        if (realizerGetter.mustGet(line)) {
            line = realizerGetter.performOperations(line, this);

            String value = realizerGetter.getValue(line);

            if (value != null) {
                String[] nomPrenom = value.split(" ");

                StringBuilder nom = new StringBuilder(25);
                String prenom = StringUtils.setFirstLetterUpper(nomPrenom[0]);

                for (int i = 1; i < nomPrenom.length; ++i) {
                    nom.append(StringUtils.setFirstLetterUpper(nomPrenom[i]));
                    nom.append(' ');
                }

                StringUtils.removeLastSpace(nom);

                if (realizersService.exists(prenom, nom.toString())) {
                    getFilm().setTheRealizer(realizersService.getRealizer(prenom, nom.toString()));
                } else {
                    RealizerImpl realisateur = new RealizerImpl();
                    realisateur.setName(nom.toString());
                    realisateur.setFirstName(prenom);
                    realisateur.setTheCountry(countriesService.getDefaultCountry());
                    realisateur.setNote(notesService.getDefaultNote());

                    realizersService.create(realisateur);

                    getFilm().setTheRealizer(realisateur);
                }

                setRealizer(true);
            }
        }
    }

    @Override
    public void findActors(String line) {
        if (isActorsDo()) {
            return;
        }

        if (actorsGetter.mustGet(line)) {
            line = actorsGetter.performOperations(line, this);

            String value = actorsGetter.getValue(line);

            if (value != null) {
                Set<ActorImpl> actors = new HashSet<ActorImpl>(filmsModule.getConfiguration().getNumberOfActors());

                value = StringUtils.removeHTMLEntities(value);

                //On dcoupe la chaine en autant qu'il y a d'acteurs dedans
                String[] actorsTemp = value.split("%%%");

                int current = 0;

                //On parcours la listes des acteurs
                for (String name : actorsTemp) {
                    if (current >= filmsModule.getConfiguration().getNumberOfActors()) {
                        break;
                    }

                    current++;

                    //On spare le nom en parties
                    String[] nomPrenom = name.split(" ");

                    StringBuilder nom = new StringBuilder(25);
                    String prenom = nomPrenom[0];

                    //On spare le nom et le prnom du ralisateur
                    for (int i = 1; i < nomPrenom.length; ++i) {
                        nom.append(StringUtils.setFirstLetterUpper(nomPrenom[i]));
                        nom.append(' ');
                    }

                    StringUtils.removeLastSpace(nom);

                    //Si cet acteur n'existe pas, on le cre
                    if (actorService.exist(prenom, nom.toString())) {
                        ActorImpl actor = actorService.getActor(prenom, nom.toString());
                        actors.add(actor);
                    } else {
                        ActorImpl actor = new ActorImpl();
                        actor.setName(nom.toString());
                        actor.setFirstName(prenom);
                        actor.setTheCountry(countriesService.getDefaultCountry());
                        actor.setNote(notesService.getDefaultNote());

                        actorService.create(actor);

                        actors.add(actor);
                    }

                    if (actors.size() >= filmsModule.getConfiguration().getNumberOfActors()) {
                        break;
                    }
                }

                getFilm().setActors(actors);
            }
        }
    }

    @Override
    public void findResume(String line) {
        if (isResumeDo()) {
            return;
        }

        if (resumeGetter.mustGet(line)) {
            line = resumeGetter.performOperations(line, this);

            String value = resumeGetter.getValue(line);

            if (value != null) {
                getFilm().setResume(StringUtils.removeHTMLEntities(value));

                setResume(true);
            }
        }
    }
}